@@ -157,9 +157,12 @@ impl<T> CpuAccessibleBuffer<[T]> {
         data: I,
     ) -> Result<Arc<CpuAccessibleBuffer<[T]>>, DeviceMemoryAllocError>
     where
-        I: ExactSizeIterator<Item = T>,
+        I: IntoIterator<Item = T>,
+        I::IntoIter: ExactSizeIterator,
         T: Content + 'static,
     {
+        let data = data.into_iter();
+
         unsafe {
             let uninitialized = CpuAccessibleBuffer::uninitialized_array(
                 device,
@@ -367,7 +370,8 @@ where
 
 unsafe impl<T: ?Sized, A> BufferAccess for CpuAccessibleBuffer<T, A>
 where
-    T: 'static + Send + Sync,
+    T: Send + Sync + 'static,
+    A: Send + Sync,
 {
     #[inline]
     fn inner(&self) -> BufferInner {
@@ -483,7 +487,8 @@ where
 
 unsafe impl<T: ?Sized, A> TypedBufferAccess for CpuAccessibleBuffer<T, A>
 where
-    T: 'static + Send + Sync,
+    T: Send + Sync + 'static,
+    A: Send + Sync,
 {
     type Content = T;
 }
@@ -497,7 +502,8 @@ unsafe impl<T: ?Sized, A> DeviceOwned for CpuAccessibleBuffer<T, A> {
 
 impl<T: ?Sized, A> PartialEq for CpuAccessibleBuffer<T, A>
 where
-    T: 'static + Send + Sync,
+    T: Send + Sync + 'static,
+    A: Send + Sync,
 {
     #[inline]
     fn eq(&self, other: &Self) -> bool {
@@ -505,11 +511,17 @@ where
     }
 }
 
-impl<T: ?Sized, A> Eq for CpuAccessibleBuffer<T, A> where T: 'static + Send + Sync {}
+impl<T: ?Sized, A> Eq for CpuAccessibleBuffer<T, A>
+where
+    T: Send + Sync + 'static,
+    A: Send + Sync,
+{
+}
 
 impl<T: ?Sized, A> Hash for CpuAccessibleBuffer<T, A>
 where
-    T: 'static + Send + Sync,
+    T: Send + Sync + 'static,
+    A: Send + Sync,
 {
     #[inline]
     fn hash<H: Hasher>(&self, state: &mut H) {
