@@ -420,11 +420,14 @@ where
     fn try_next_impl<I>(
         &self,
         cur_buf_mutex: &mut MutexGuard<Option<Arc<ActualBuffer<A>>>>,
-        mut data: I,
-    ) -> Result<CpuBufferPoolChunk<T, A>, I>
+        data: I,
+    ) -> Result<CpuBufferPoolChunk<T, A>, I::IntoIter>
     where
-        I: ExactSizeIterator<Item = T>,
+        I: IntoIterator<Item = T>,
+        I::IntoIter: ExactSizeIterator,
     {
+        let mut data = data.into_iter();
+
         // Grab the current buffer. Return `Err` if the pool wasn't "initialized" yet.
         let current_buffer = match cur_buf_mutex.clone() {
             Some(b) => b,
@@ -620,6 +623,7 @@ where
 
 unsafe impl<T, A> BufferAccess for CpuBufferPoolChunk<T, A>
 where
+    T: Send + Sync,
     A: MemoryPool,
 {
     #[inline]
@@ -731,6 +735,7 @@ where
 
 unsafe impl<T, A> TypedBufferAccess for CpuBufferPoolChunk<T, A>
 where
+    T: Send + Sync,
     A: MemoryPool,
 {
     type Content = [T];
@@ -748,6 +753,7 @@ where
 
 impl<T, A> PartialEq for CpuBufferPoolChunk<T, A>
 where
+    T: Send + Sync,
     A: MemoryPool,
 {
     #[inline]
@@ -756,10 +762,16 @@ where
     }
 }
 
-impl<T, A> Eq for CpuBufferPoolChunk<T, A> where A: MemoryPool {}
+impl<T, A> Eq for CpuBufferPoolChunk<T, A>
+where
+    T: Send + Sync,
+    A: MemoryPool,
+{
+}
 
 impl<T, A> Hash for CpuBufferPoolChunk<T, A>
 where
+    T: Send + Sync,
     A: MemoryPool,
 {
     #[inline]
@@ -782,6 +794,7 @@ where
 
 unsafe impl<T, A> BufferAccess for CpuBufferPoolSubbuffer<T, A>
 where
+    T: Send + Sync,
     A: MemoryPool,
 {
     #[inline]
@@ -817,6 +830,7 @@ where
 
 unsafe impl<T, A> TypedBufferAccess for CpuBufferPoolSubbuffer<T, A>
 where
+    T: Send + Sync,
     A: MemoryPool,
 {
     type Content = T;
@@ -834,6 +848,7 @@ where
 
 impl<T, A> PartialEq for CpuBufferPoolSubbuffer<T, A>
 where
+    T: Send + Sync,
     A: MemoryPool,
 {
     #[inline]
@@ -842,10 +857,16 @@ where
     }
 }
 
-impl<T, A> Eq for CpuBufferPoolSubbuffer<T, A> where A: MemoryPool {}
+impl<T, A> Eq for CpuBufferPoolSubbuffer<T, A>
+where
+    T: Send + Sync,
+    A: MemoryPool,
+{
+}
 
 impl<T, A> Hash for CpuBufferPoolSubbuffer<T, A>
 where
+    T: Send + Sync,
     A: MemoryPool,
 {
     #[inline]
