@@ -96,7 +96,7 @@ impl<T: ?Sized> ImmutableBuffer<T> {
         queue: Arc<Queue>,
     ) -> Result<(Arc<ImmutableBuffer<T>>, ImmutableBufferFromBufferFuture), DeviceMemoryAllocError>
     where
-        T: 'static + Copy + Send + Sync + Sized,
+        T: Copy + Send + Sync + Sized + 'static,
     {
         let source = CpuAccessibleBuffer::from_data(
             queue.device().clone(),
@@ -119,8 +119,8 @@ impl<T: ?Sized> ImmutableBuffer<T> {
         queue: Arc<Queue>,
     ) -> Result<(Arc<ImmutableBuffer<T>>, ImmutableBufferFromBufferFuture), DeviceMemoryAllocError>
     where
-        B: BufferAccess + TypedBufferAccess<Content = T> + 'static + Clone + Send + Sync,
-        T: 'static + Send + Sync,
+        B: TypedBufferAccess<Content = T> + Clone + 'static,
+        T: Send + Sync + 'static,
     {
         unsafe {
             // We automatically set `transfer_destination` to true in order to avoid annoying errors.
@@ -193,8 +193,9 @@ impl<T> ImmutableBuffer<[T]> {
         queue: Arc<Queue>,
     ) -> Result<(Arc<ImmutableBuffer<[T]>>, ImmutableBufferFromBufferFuture), DeviceMemoryAllocError>
     where
-        D: ExactSizeIterator<Item = T>,
-        T: 'static + Send + Sync + Sized,
+        D: IntoIterator<Item = T>,
+        D::IntoIter: ExactSizeIterator,
+        T: Send + Sync + Sized + 'static,
     {
         let source = CpuAccessibleBuffer::from_iter(
             queue.device().clone(),
@@ -353,7 +354,11 @@ impl<T: ?Sized, A> ImmutableBuffer<T, A> {
     }
 }
 
-unsafe impl<T: ?Sized, A> BufferAccess for ImmutableBuffer<T, A> {
+unsafe impl<T, A> BufferAccess for ImmutableBuffer<T, A>
+where
+    T: Send + Sync + ?Sized,
+    A: Send + Sync,
+{
     #[inline]
     fn inner(&self) -> BufferInner {
         BufferInner {
@@ -392,7 +397,11 @@ unsafe impl<T: ?Sized, A> BufferAccess for ImmutableBuffer<T, A> {
     unsafe fn unlock(&self) {}
 }
 
-unsafe impl<T: ?Sized, A> TypedBufferAccess for ImmutableBuffer<T, A> {
+unsafe impl<T, A> TypedBufferAccess for ImmutableBuffer<T, A>
+where
+    T: Send + Sync + ?Sized,
+    A: Send + Sync,
+{
     type Content = T;
 }
 
@@ -403,16 +412,29 @@ unsafe impl<T: ?Sized, A> DeviceOwned for ImmutableBuffer<T, A> {
     }
 }
 
-impl<T: ?Sized, A> PartialEq for ImmutableBuffer<T, A> {
+impl<T: ?Sized, A> PartialEq for ImmutableBuffer<T, A>
+where
+    T: Send + Sync,
+    A: Send + Sync,
+{
     #[inline]
     fn eq(&self, other: &Self) -> bool {
         self.inner() == other.inner() && self.size() == other.size()
     }
 }
 
-impl<T: ?Sized, A> Eq for ImmutableBuffer<T, A> {}
+impl<T: ?Sized, A> Eq for ImmutableBuffer<T, A>
+where
+    T: Send + Sync,
+    A: Send + Sync,
+{
+}
 
-impl<T: ?Sized, A> Hash for ImmutableBuffer<T, A> {
+impl<T: ?Sized, A> Hash for ImmutableBuffer<T, A>
+where
+    T: Send + Sync,
+    A: Send + Sync,
+{
     #[inline]
     fn hash<H: Hasher>(&self, state: &mut H) {
         self.inner().hash(state);
@@ -430,7 +452,11 @@ pub struct ImmutableBufferInitialization<
     used: Arc<AtomicBool>,
 }
 
-unsafe impl<T: ?Sized, A> BufferAccess for ImmutableBufferInitialization<T, A> {
+unsafe impl<T, A> BufferAccess for ImmutableBufferInitialization<T, A>
+where
+    T: Send + Sync + ?Sized,
+    A: Send + Sync,
+{
     #[inline]
     fn inner(&self) -> BufferInner {
         self.buffer.inner()
@@ -474,7 +500,11 @@ unsafe impl<T: ?Sized, A> BufferAccess for ImmutableBufferInitialization<T, A> {
     }
 }
 
-unsafe impl<T: ?Sized, A> TypedBufferAccess for ImmutableBufferInitialization<T, A> {
+unsafe impl<T, A> TypedBufferAccess for ImmutableBufferInitialization<T, A>
+where
+    T: Send + Sync + ?Sized,
+    A: Send + Sync,
+{
     type Content = T;
 }
 
@@ -495,16 +525,29 @@ impl<T: ?Sized, A> Clone for ImmutableBufferInitialization<T, A> {
     }
 }
 
-impl<T: ?Sized, A> PartialEq for ImmutableBufferInitialization<T, A> {
+impl<T: ?Sized, A> PartialEq for ImmutableBufferInitialization<T, A>
+where
+    T: Send + Sync,
+    A: Send + Sync,
+{
     #[inline]
     fn eq(&self, other: &Self) -> bool {
         self.inner() == other.inner() && self.size() == other.size()
     }
 }
 
-impl<T: ?Sized, A> Eq for ImmutableBufferInitialization<T, A> {}
+impl<T: ?Sized, A> Eq for ImmutableBufferInitialization<T, A>
+where
+    T: Send + Sync,
+    A: Send + Sync,
+{
+}
 
-impl<T: ?Sized, A> Hash for ImmutableBufferInitialization<T, A> {
+impl<T: ?Sized, A> Hash for ImmutableBufferInitialization<T, A>
+where
+    T: Send + Sync,
+    A: Send + Sync,
+{
     #[inline]
     fn hash<H: Hasher>(&self, state: &mut H) {
         self.inner().hash(state);
