@@ -16,16 +16,13 @@
 //! trait. By default vulkano will use the `StandardCommandPool` struct, but you can implement
 //! this trait yourself by wrapping around the `UnsafeCommandPool` type.
 
-use crate::device::physical::QueueFamily;
-
-use crate::device::DeviceOwned;
-use crate::OomError;
-
 pub use self::standard::StandardCommandPool;
 pub use self::sys::CommandPoolTrimError;
 pub use self::sys::UnsafeCommandPool;
 pub use self::sys::UnsafeCommandPoolAlloc;
-pub use self::sys::UnsafeCommandPoolAllocIter;
+use crate::device::physical::QueueFamily;
+use crate::device::DeviceOwned;
+use crate::OomError;
 
 pub mod standard;
 mod sys;
@@ -94,7 +91,7 @@ pub unsafe trait CommandPoolBuilderAlloc: DeviceOwned {
 ///
 /// See `CommandPool` for information about safety.
 ///
-pub unsafe trait CommandPoolAlloc: DeviceOwned {
+pub unsafe trait CommandPoolAlloc: DeviceOwned + Send + Sync {
     /// Returns the internal object that contains the command buffer.
     fn inner(&self) -> &UnsafeCommandPoolAlloc;
 
