@@ -13,17 +13,15 @@
 //! can create a descriptor set layout manually, but it is normally created automatically by each
 //! pipeline layout.
 
-pub use self::desc::DescriptorBufferDesc;
+pub use self::desc::DescriptorCompatibilityError;
 pub use self::desc::DescriptorDesc;
-pub use self::desc::DescriptorDescSupersetError;
+pub use self::desc::DescriptorDescImage;
 pub use self::desc::DescriptorDescTy;
-pub use self::desc::DescriptorImageDesc;
-pub use self::desc::DescriptorImageDescArray;
-pub use self::desc::DescriptorImageDescDimensions;
+pub use self::desc::DescriptorSetCompatibilityError;
 pub use self::desc::DescriptorSetDesc;
-pub use self::desc::DescriptorSetDescSupersetError;
 pub use self::desc::DescriptorType;
 pub use self::sys::DescriptorSetLayout;
+pub use self::sys::DescriptorSetLayoutError;
 
 mod desc;
 mod sys;
