@@ -10,7 +10,6 @@
 use crate::device::Device;
 use crate::format::ClearValue;
 use crate::format::Format;
-use crate::format::FormatTy;
 use crate::image::sys::ImageCreationError;
 use crate::image::sys::UnsafeImage;
 use crate::image::traits::ImageAccess;
@@ -23,6 +22,14 @@ use crate::image::ImageInner;
 use crate::image::ImageLayout;
 use crate::image::ImageUsage;
 use crate::image::SampleCount;
+#[cfg(any(
+    target_os = "linux",
+    target_os = "dragonflybsd",
+    target_os = "freebsd",
+    target_os = "netbsd",
+    target_os = "openbsd"
+))]
+use crate::memory::pool::alloc_dedicated_with_exportable_fd;
 use crate::memory::pool::AllocFromRequirementsFilter;
 use crate::memory::pool::AllocLayout;
 use crate::memory::pool::MappingRequirement;
@@ -31,8 +38,32 @@ use crate::memory::pool::MemoryPoolAlloc;
 use crate::memory::pool::PotentialDedicatedAllocation;
 use crate::memory::pool::StdMemoryPoolAlloc;
 use crate::memory::DedicatedAlloc;
+#[cfg(any(
+    target_os = "linux",
+    target_os = "dragonflybsd",
+    target_os = "freebsd",
+    target_os = "netbsd",
+    target_os = "openbsd"
+))]
+use crate::memory::{DeviceMemoryAllocError, ExternalMemoryHandleType};
 use crate::sync::AccessError;
 use crate::sync::Sharing;
+#[cfg(any(
+    target_os = "linux",
+    target_os = "dragonflybsd",
+    target_os = "freebsd",
+    target_os = "netbsd",
+    target_os = "openbsd"
+))]
+use crate::DeviceSize;
+#[cfg(any(
+    target_os = "linux",
+    target_os = "dragonflybsd",
+    target_os = "freebsd",
+    target_os = "netbsd",
+    target_os = "openbsd"
+))]
+use std::fs::File;
 use std::hash::Hash;
 use std::hash::Hasher;
 use std::iter::Empty;
@@ -410,13 +441,12 @@ impl AttachmentImage {
     ) -> Result<Arc<AttachmentImage>, ImageCreationError> {
         // TODO: check dimensions against the max_framebuffer_width/height/layers limits
 
-        let is_depth = match format.ty() {
-            FormatTy::Depth => true,
-            FormatTy::DepthStencil => true,
-            FormatTy::Stencil => true,
-            FormatTy::Compressed => panic!(),
-            _ => false,
-        };
+        let aspects = format.aspects();
+        let is_depth = aspects.depth || aspects.stencil;
+
+        if format.compression().is_some() {
+            panic!() // TODO: message?
+        }
 
         let usage = ImageUsage {
             color_attachment: !is_depth,
@@ -477,18 +507,130 @@ impl AttachmentImage {
             gpu_lock: AtomicUsize::new(0),
         }))
     }
-}
 
-impl<A> AttachmentImage<A> {
-    /// Returns the dimensions of the image.
-    #[inline]
-    pub fn dimensions(&self) -> [u32; 2] {
-        let dims = self.image.dimensions();
-        [dims.width(), dims.height()]
+    #[cfg(any(
+        target_os = "linux",
+        target_os = "dragonflybsd",
+        target_os = "freebsd",
+        target_os = "netbsd",
+        target_os = "openbsd"
+    ))]
+    pub fn new_with_exportable_fd(
+        device: Arc<Device>,
+        dimensions: [u32; 2],
+        array_layers: u32,
+        format: Format,
+        base_usage: ImageUsage,
+        samples: SampleCount,
+    ) -> Result<Arc<AttachmentImage>, ImageCreationError> {
+        // TODO: check dimensions against the max_framebuffer_width/height/layers limits
+
+        let aspects = format.aspects();
+        let is_depth = aspects.depth || aspects.stencil;
+
+        let usage = ImageUsage {
+            color_attachment: !is_depth,
+            depth_stencil_attachment: is_depth,
+            ..base_usage
+        };
+
+        let (image, mem_reqs) = unsafe {
+            let dims = ImageDimensions::Dim2d {
+                width: dimensions[0],
+                height: dimensions[1],
+                array_layers,
+            };
+
+            UnsafeImage::new(
+                device.clone(),
+                usage,
+                format,
+                ImageCreateFlags::none(),
+                dims,
+                samples,
+                1,
+                Sharing::Exclusive::<Empty<u32>>,
+                false,
+                false,
+            )?
+        };
+
+        let memory = alloc_dedicated_with_exportable_fd(
+            device.clone(),
+            &mem_reqs,
+            AllocLayout::Optimal,
+            MappingRequirement::DoNotMap,
+            DedicatedAlloc::Image(&image),
+            |t| {
+                if t.is_device_local() {
+                    AllocFromRequirementsFilter::Preferred
+                } else {
+                    AllocFromRequirementsFilter::Allowed
+                }
+            },
+        )?;
+
+        debug_assert!((memory.offset() % mem_reqs.alignment) == 0);
+        unsafe {
+            image.bind_memory(memory.memory(), memory.offset())?;
+        }
+
+        Ok(Arc::new(AttachmentImage {
+            image,
+            memory,
+            format,
+            attachment_layout: if is_depth {
+                ImageLayout::DepthStencilAttachmentOptimal
+            } else {
+                ImageLayout::ColorAttachmentOptimal
+            },
+            initialized: AtomicBool::new(false),
+            gpu_lock: AtomicUsize::new(0),
+        }))
+    }
+
+    /// Exports posix file descriptor for the allocated memory
+    /// requires `khr_external_memory_fd` and `khr_external_memory` extensions to be loaded.
+    /// Only works on Linux.
+    #[cfg(any(
+        target_os = "linux",
+        target_os = "dragonflybsd",
+        target_os = "freebsd",
+        target_os = "netbsd",
+        target_os = "openbsd"
+    ))]
+    pub fn export_posix_fd(&self) -> Result<File, DeviceMemoryAllocError> {
+        self.memory
+            .memory()
+            .export_fd(ExternalMemoryHandleType::posix())
+    }
+
+    /// Return the size of the allocated memory (used for e.g. with cuda)
+    #[cfg(any(
+        target_os = "linux",
+        target_os = "dragonflybsd",
+        target_os = "freebsd",
+        target_os = "netbsd",
+        target_os = "openbsd"
+    ))]
+    pub fn mem_size(&self) -> DeviceSize {
+        self.memory.memory().size()
     }
 }
 
-unsafe impl<A> ImageAccess for AttachmentImage<A> {
+impl<A> AttachmentImage<A> {
+    /// Returns the width, height and layers of the image.
+    #[inline]
+    pub fn dimensions(&self) -> [u32; 3] {
+        let dims = self.image.dimensions();
+        [dims.width(), dims.height(), dims.array_layers()]
+    }
+}
+
+unsafe impl<A> ImageAccess for AttachmentImage<A>
+where
+    A: MemoryPoolAlloc,
+{
     #[inline]
     fn inner(&self) -> ImageInner {
         ImageInner {
@@ -600,37 +742,49 @@ unsafe impl<A> ImageAccess for AttachmentImage<A> {
 
     #[inline]
     fn current_layer_levels_access(&self) -> std::ops::Range<u32> {
-        0..1
+        0..self.dimensions()[2]
     }
 }
 
-unsafe impl<A> ImageClearValue<ClearValue> for Arc<AttachmentImage<A>> {
+unsafe impl<A> ImageClearValue<ClearValue> for Arc<AttachmentImage<A>>
+where
+    A: MemoryPoolAlloc,
+{
     #[inline]
     fn decode(&self, value: ClearValue) -> Option<ClearValue> {
         Some(self.format.decode_clear_value(value))
     }
 }
 
-unsafe impl<P, A> ImageContent<P> for Arc<AttachmentImage<A>> {
+unsafe impl<P, A> ImageContent<P> for Arc<AttachmentImage<A>>
+where
+    A: MemoryPoolAlloc,
+{
     #[inline]
     fn matches_format(&self) -> bool {
         true // FIXME:
     }
 }
 
-impl<A> PartialEq for AttachmentImage<A> {
+impl<A> PartialEq for AttachmentImage<A>
+where
+    A: MemoryPoolAlloc,
+{
     #[inline]
     fn eq(&self, other: &Self) -> bool {
-        ImageAccess::inner(self) == ImageAccess::inner(other)
+        self.inner() == other.inner()
     }
 }
 
-impl<A> Eq for AttachmentImage<A> {}
+impl<A> Eq for AttachmentImage<A> where A: MemoryPoolAlloc {}
 
-impl<A> Hash for AttachmentImage<A> {
+impl<A> Hash for AttachmentImage<A>
+where
+    A: MemoryPoolAlloc,
+{
     #[inline]
     fn hash<H: Hasher>(&self, state: &mut H) {
-        ImageAccess::inner(self).hash(state);
+        self.inner().hash(state);
     }
 }
 
@@ -642,18 +796,18 @@ mod tests {
     #[test]
     fn create_regular() {
         let (device, _) = gfx_dev_and_queue!();
-        let _img = AttachmentImage::new(device, [32, 32], Format::R8G8B8A8Unorm).unwrap();
+        let _img = AttachmentImage::new(device, [32, 32], Format::R8G8B8A8_UNORM).unwrap();
     }
 
     #[test]
     fn create_transient() {
         let (device, _) = gfx_dev_and_queue!();
-        let _img = AttachmentImage::transient(device, [32, 32], Format::R8G8B8A8Unorm).unwrap();
+        let _img = AttachmentImage::transient(device, [32, 32], Format::R8G8B8A8_UNORM).unwrap();
     }
 
     #[test]
     fn d16_unorm_always_supported() {
         let (device, _) = gfx_dev_and_queue!();
-        let _img = AttachmentImage::new(device, [32, 32], Format::D16Unorm).unwrap();
+        let _img = AttachmentImage::new(device, [32, 32], Format::D16_UNORM).unwrap();
     }
 }
