@@ -9,7 +9,6 @@
 
 use crate::format::ClearValue;
 use crate::format::Format;
-use crate::format::FormatTy;
 use crate::image::sys::UnsafeImage;
 use crate::image::ImageDescriptorLayouts;
 use crate::image::ImageDimensions;
@@ -21,7 +20,7 @@ use std::hash::Hash;
 use std::hash::Hasher;
 
 /// Trait for types that represent the way a GPU can access an image.
-pub unsafe trait ImageAccess {
+pub unsafe trait ImageAccess: Send + Sync {
     /// Returns the inner unsafe image object used by this image.
     fn inner(&self) -> ImageInner;
 
@@ -31,32 +30,6 @@ pub unsafe trait ImageAccess {
         self.inner().image.format()
     }
 
-    /// Returns true if the image is a color image.
-    #[inline]
-    fn has_color(&self) -> bool {
-        matches!(
-            self.format().ty(),
-            FormatTy::Float | FormatTy::Uint | FormatTy::Sint | FormatTy::Compressed
-        )
-    }
-
-    /// Returns true if the image has a depth component. In other words, if it is a depth or a
-    /// depth-stencil format.
-    #[inline]
-    fn has_depth(&self) -> bool {
-        matches!(self.format().ty(), FormatTy::Depth | FormatTy::DepthStencil)
-    }
-
-    /// Returns true if the image has a stencil component. In other words, if it is a stencil or a
-    /// depth-stencil format.
-    #[inline]
-    fn has_stencil(&self) -> bool {
-        matches!(
-            self.format().ty(),
-            FormatTy::Stencil | FormatTy::DepthStencil
-        )
-    }
-
     /// Returns the number of mipmap levels of this image.
     #[inline]
     fn mipmap_levels(&self) -> u32 {
@@ -249,7 +222,7 @@ pub struct ImageInner<'a> {
 
 unsafe impl<T> ImageAccess for T
 where
-    T: SafeDeref,
+    T: SafeDeref + Send + Sync,
     T::Target: ImageAccess,
 {
     #[inline]
@@ -316,16 +289,16 @@ where
     }
 }
 
-impl PartialEq for dyn ImageAccess + Send + Sync {
+impl PartialEq for dyn ImageAccess {
     #[inline]
     fn eq(&self, other: &Self) -> bool {
         self.inner() == other.inner()
     }
 }
 
-impl Eq for dyn ImageAccess + Send + Sync {}
+impl Eq for dyn ImageAccess {}
 
-impl Hash for dyn ImageAccess + Send + Sync {
+impl Hash for dyn ImageAccess {
     #[inline]
     fn hash<H: Hasher>(&self, state: &mut H) {
         self.inner().hash(state);
