@@ -21,7 +21,13 @@ use crate::Version;
 use crate::VulkanObject;
 use std::error;
 use std::fmt;
-#[cfg(target_os = "linux")]
+#[cfg(any(
+    target_os = "linux",
+    target_os = "dragonflybsd",
+    target_os = "freebsd",
+    target_os = "netbsd",
+    target_os = "openbsd"
+))]
 use std::fs::File;
 use std::marker::PhantomData;
 use std::mem::MaybeUninit;
@@ -29,7 +35,13 @@ use std::ops::Deref;
 use std::ops::DerefMut;
 use std::ops::Range;
 use std::os::raw::c_void;
-#[cfg(target_os = "linux")]
+#[cfg(any(
+    target_os = "linux",
+    target_os = "dragonflybsd",
+    target_os = "freebsd",
+    target_os = "netbsd",
+    target_os = "openbsd"
+))]
 use std::os::unix::io::{FromRawFd, IntoRawFd};
 use std::ptr;
 use std::sync::Arc;
@@ -190,7 +202,13 @@ impl<'a> DeviceMemoryBuilder<'a> {
     /// # Panic
     ///
     /// - Panics if the import info has already been set.
-    #[cfg(target_os = "linux")]
+    #[cfg(any(
+        target_os = "linux",
+        target_os = "dragonflybsd",
+        target_os = "freebsd",
+        target_os = "netbsd",
+        target_os = "openbsd"
+    ))]
     pub fn import_info(
         mut self,
         fd: File,
@@ -340,11 +358,7 @@ impl<'a> DeviceMemoryBuilder<'a> {
                 .lock()
                 .expect("Poisoned mutex");
 
-            if *allocation_count
-                >= physical_device
-                    .properties()
-                    .max_memory_allocation_count
-            {
+            if *allocation_count >= physical_device.properties().max_memory_allocation_count {
                 return Err(DeviceMemoryAllocError::TooManyObjects);
             }
             let fns = self.device.fns();
@@ -448,9 +462,15 @@ impl DeviceMemory {
         Self::map_allocation(device.clone(), mem)
     }
 
-    /// Same as `alloc`, but allows exportable file descriptor on Linux.
+    /// Same as `alloc`, but allows exportable file descriptor on Linux/BSD.
     #[inline]
-    #[cfg(target_os = "linux")]
+    #[cfg(any(
+        target_os = "linux",
+        target_os = "dragonflybsd",
+        target_os = "freebsd",
+        target_os = "netbsd",
+        target_os = "openbsd"
+    ))]
     pub fn alloc_with_exportable_fd(
         device: Arc<Device>,
         memory_type: MemoryType,
@@ -468,9 +488,15 @@ impl DeviceMemory {
         Ok(Arc::try_unwrap(memory).unwrap())
     }
 
-    /// Same as `dedicated_alloc`, but allows exportable file descriptor on Linux.
+    /// Same as `dedicated_alloc`, but allows exportable file descriptor on Linux/BSD.
     #[inline]
-    #[cfg(target_os = "linux")]
+    #[cfg(any(
+        target_os = "linux",
+        target_os = "dragonflybsd",
+        target_os = "freebsd",
+        target_os = "netbsd",
+        target_os = "openbsd"
+    ))]
     pub fn dedicated_alloc_with_exportable_fd(
         device: Arc<Device>,
         memory_type: MemoryType,
@@ -490,9 +516,15 @@ impl DeviceMemory {
         Ok(Arc::try_unwrap(memory).unwrap())
     }
 
-    /// Same as `alloc_and_map`, but allows exportable file descriptor on Linux.
+    /// Same as `alloc_and_map`, but allows exportable file descriptor on Linux/BSD.
     #[inline]
-    #[cfg(target_os = "linux")]
+    #[cfg(any(
+        target_os = "linux",
+        target_os = "dragonflybsd",
+        target_os = "freebsd",
+        target_os = "netbsd",
+        target_os = "openbsd"
+    ))]
     pub fn alloc_and_map_with_exportable_fd(
         device: Arc<Device>,
         memory_type: MemoryType,
@@ -506,9 +538,15 @@ impl DeviceMemory {
         )
     }
 
-    /// Same as `dedicated_alloc_and_map`, but allows exportable file descriptor on Linux.
+    /// Same as `dedicated_alloc_and_map`, but allows exportable file descriptor on Linux/BSD.
     #[inline]
-    #[cfg(target_os = "linux")]
+    #[cfg(any(
+        target_os = "linux",
+        target_os = "dragonflybsd",
+        target_os = "freebsd",
+        target_os = "netbsd",
+        target_os = "openbsd"
+    ))]
     pub fn dedicated_alloc_and_map_with_exportable_fd(
         device: Arc<Device>,
         memory_type: MemoryType,
@@ -576,7 +614,13 @@ impl DeviceMemory {
     ///
     /// - Panics if the user requests an invalid handle type for this device memory object.
     #[inline]
-    #[cfg(target_os = "linux")]
+    #[cfg(any(
+        target_os = "linux",
+        target_os = "dragonflybsd",
+        target_os = "freebsd",
+        target_os = "netbsd",
+        target_os = "openbsd"
+    ))]
     pub fn export_fd(
         &self,
         handle_type: ExternalMemoryHandleType,
@@ -594,7 +638,8 @@ impl DeviceMemory {
 
         // VUID-VkMemoryGetFdInfoKHR-handleType-00671: "handleType must have been included in
         // VkExportMemoryAllocateInfo::handleTypes when memory was created".
-        if (bits & ash::vk::ExternalMemoryHandleTypeFlags::from(self.handle_types)).is_empty() {
+        let self_bits = ash::vk::ExternalMemoryHandleTypeFlags::from(self.handle_types);
+        if (bits & self_bits).is_empty() {
             return Err(DeviceMemoryAllocError::SpecViolation(671))?;
         }
 
