@@ -7,7 +7,6 @@
 // notice may not be copied, modified, or distributed except
 // according to those terms.
 
-use crate::buffer::BufferAccess;
 use crate::format::Format;
 use crate::pipeline::shader::ShaderInterface;
 use crate::pipeline::vertex::VertexInput;
@@ -15,12 +14,9 @@ use crate::pipeline::vertex::VertexMemberTy;
 use crate::SafeDeref;
 use std::error;
 use std::fmt;
-use std::sync::Arc;
 
 /// Trait for types that describe the definition of the vertex input used by a graphics pipeline.
-pub unsafe trait VertexDefinition:
-    VertexSource<Vec<Arc<dyn BufferAccess + Send + Sync>>>
-{
+pub unsafe trait VertexDefinition {
     /// Builds the vertex definition to use to link this definition to a vertex shader's input
     /// interface.
     // TODO: remove error return, move checks to GraphicsPipelineBuilder
@@ -82,24 +78,3 @@ impl fmt::Display for IncompatibleVertexDefinitionError {
         )
     }
 }
-
-/// Extension trait of `VertexDefinition`. The `L` parameter is an acceptable vertex source for this
-/// vertex definition.
-pub unsafe trait VertexSource<L> {
-    /// Checks and returns the list of buffers with offsets, number of vertices and number of instances.
-    // TODO: return error if problem
-    // TODO: better than a Vec
-    // TODO: return a struct instead
-    fn decode(&self, list: L) -> (Vec<Box<dyn BufferAccess + Send + Sync>>, usize, usize);
-}
-
-unsafe impl<L, T> VertexSource<L> for T
-where
-    T: SafeDeref,
-    T::Target: VertexSource<L>,
-{
-    #[inline]
-    fn decode(&self, list: L) -> (Vec<Box<dyn BufferAccess + Send + Sync>>, usize, usize) {
-        (**self).decode(list)
-    }
-}
