@@ -48,7 +48,7 @@ use std::sync::Arc;
 /// use vulkano::render_pass::Framebuffer;
 ///
 /// # let render_pass: Arc<RenderPass> = return;
-/// # let view: Arc<vulkano::image::view::ImageView<Arc<vulkano::image::AttachmentImage<vulkano::format::Format>>>> = return;
+/// # let view: Arc<vulkano::image::view::ImageView<Arc<vulkano::image::AttachmentImage>>> = return;
 /// // let render_pass: Arc<_> = ...;
 /// let framebuffer = Framebuffer::start(render_pass.clone())
 ///     .add(view).unwrap()
@@ -56,7 +56,7 @@ use std::sync::Arc;
 /// ```
 ///
 /// All framebuffer objects implement the `FramebufferAbstract` trait. This means that you can cast
-/// any `Arc<Framebuffer<..>>` into an `Arc<FramebufferAbstract + Send + Sync>` for easier storage.
+/// any `Arc<Framebuffer<..>>` into an `Arc<FramebufferAbstract>` for easier storage.
 ///
 /// ## Framebuffer dimensions
 ///
@@ -387,8 +387,8 @@ impl<A> Framebuffer<A> {
 /// Trait for objects that contain a Vulkan framebuffer object.
 ///
 /// Any `Framebuffer` object implements this trait. You can therefore turn a `Arc<Framebuffer<_>>`
-/// into a `Arc<FramebufferAbstract + Send + Sync>` for easier storage.
-pub unsafe trait FramebufferAbstract {
+/// into a `Arc<FramebufferAbstract>` for easier storage.
+pub unsafe trait FramebufferAbstract: Send + Sync {
     /// Returns an opaque struct that represents the framebuffer's internals.
     fn inner(&self) -> FramebufferSys;
 
@@ -424,12 +424,12 @@ pub unsafe trait FramebufferAbstract {
 
 unsafe impl<T> FramebufferAbstract for T
 where
-    T: SafeDeref,
+    T: SafeDeref + Send + Sync,
     T::Target: FramebufferAbstract,
 {
     #[inline]
     fn inner(&self) -> FramebufferSys {
-        FramebufferAbstract::inner(&**self)
+        (**self).inner()
     }
 
     #[inline]
@@ -619,7 +619,7 @@ mod tests {
                     color: {
                         load: Clear,
                         store: DontCare,
-                        format: Format::R8G8B8A8Unorm,
+                        format: Format::R8G8B8A8_UNORM,
                         samples: 1,
                     }
                 },
@@ -632,7 +632,7 @@ mod tests {
         );
 
         let view = ImageView::new(
-            AttachmentImage::new(device.clone(), [1024, 768], Format::R8G8B8A8Unorm).unwrap(),
+            AttachmentImage::new(device.clone(), [1024, 768], Format::R8G8B8A8_UNORM).unwrap(),
         )
         .unwrap();
         let _ = Framebuffer::start(render_pass)
@@ -664,7 +664,7 @@ mod tests {
                     color: {
                         load: Clear,
                         store: DontCare,
-                        format: Format::R8G8B8A8Unorm,
+                        format: Format::R8G8B8A8_UNORM,
                         samples: 1,
                     }
                 },
@@ -677,7 +677,7 @@ mod tests {
         );
 
         let view = ImageView::new(
-            AttachmentImage::new(device.clone(), [1024, 768], Format::R8Unorm).unwrap(),
+            AttachmentImage::new(device.clone(), [1024, 768], Format::R8_UNORM).unwrap(),
         )
         .unwrap();
 
@@ -699,7 +699,7 @@ mod tests {
                     color: {
                         load: Clear,
                         store: DontCare,
-                        format: Format::R8G8B8A8Unorm,
+                        format: Format::R8G8B8A8_UNORM,
                         samples: 1,
                     }
                 },
@@ -712,7 +712,7 @@ mod tests {
         );
 
         let view = ImageView::new(
-            AttachmentImage::new(device.clone(), [600, 600], Format::R8G8B8A8Unorm).unwrap(),
+            AttachmentImage::new(device.clone(), [600, 600], Format::R8G8B8A8_UNORM).unwrap(),
         )
         .unwrap();
 
@@ -733,7 +733,7 @@ mod tests {
                     color: {
                         load: Clear,
                         store: DontCare,
-                        format: Format::R8G8B8A8Unorm,
+                        format: Format::R8G8B8A8_UNORM,
                         samples: 1,
                     }
                 },
@@ -746,7 +746,7 @@ mod tests {
         );
 
         let view = ImageView::new(
-            AttachmentImage::new(device.clone(), [512, 700], Format::R8G8B8A8Unorm).unwrap(),
+            AttachmentImage::new(device.clone(), [512, 700], Format::R8G8B8A8_UNORM).unwrap(),
         )
         .unwrap();
 
@@ -772,13 +772,13 @@ mod tests {
                     a: {
                         load: Clear,
                         store: DontCare,
-                        format: Format::R8G8B8A8Unorm,
+                        format: Format::R8G8B8A8_UNORM,
                         samples: 1,
                     },
                     b: {
                         load: Clear,
                         store: DontCare,
-                        format: Format::R8G8B8A8Unorm,
+                        format: Format::R8G8B8A8_UNORM,
                         samples: 1,
                     }
                 },
@@ -791,11 +791,11 @@ mod tests {
         );
 
         let a = ImageView::new(
-            AttachmentImage::new(device.clone(), [512, 512], Format::R8G8B8A8Unorm).unwrap(),
+            AttachmentImage::new(device.clone(), [512, 512], Format::R8G8B8A8_UNORM).unwrap(),
         )
         .unwrap();
         let b = ImageView::new(
-            AttachmentImage::new(device.clone(), [512, 513], Format::R8G8B8A8Unorm).unwrap(),
+            AttachmentImage::new(device.clone(), [512, 513], Format::R8G8B8A8_UNORM).unwrap(),
         )
         .unwrap();
 
@@ -821,13 +821,13 @@ mod tests {
                     a: {
                         load: Clear,
                         store: DontCare,
-                        format: Format::R8G8B8A8Unorm,
+                        format: Format::R8G8B8A8_UNORM,
                         samples: 1,
                     },
                     b: {
                         load: Clear,
                         store: DontCare,
-                        format: Format::R8G8B8A8Unorm,
+                        format: Format::R8G8B8A8_UNORM,
                         samples: 1,
                     }
                 },
@@ -840,11 +840,11 @@ mod tests {
         );
 
         let a = ImageView::new(
-            AttachmentImage::new(device.clone(), [256, 512], Format::R8G8B8A8Unorm).unwrap(),
+            AttachmentImage::new(device.clone(), [256, 512], Format::R8G8B8A8_UNORM).unwrap(),
         )
         .unwrap();
         let b = ImageView::new(
-            AttachmentImage::new(device.clone(), [512, 128], Format::R8G8B8A8Unorm).unwrap(),
+            AttachmentImage::new(device.clone(), [512, 128], Format::R8G8B8A8_UNORM).unwrap(),
         )
         .unwrap();
 
@@ -872,13 +872,13 @@ mod tests {
                     a: {
                         load: Clear,
                         store: DontCare,
-                        format: Format::R8G8B8A8Unorm,
+                        format: Format::R8G8B8A8_UNORM,
                         samples: 1,
                     },
                     b: {
                         load: Clear,
                         store: DontCare,
-                        format: Format::R8G8B8A8Unorm,
+                        format: Format::R8G8B8A8_UNORM,
                         samples: 1,
                     }
                 },
@@ -891,7 +891,7 @@ mod tests {
         );
 
         let view = ImageView::new(
-            AttachmentImage::new(device.clone(), [256, 512], Format::R8G8B8A8Unorm).unwrap(),
+            AttachmentImage::new(device.clone(), [256, 512], Format::R8G8B8A8_UNORM).unwrap(),
         )
         .unwrap();
 
@@ -919,7 +919,7 @@ mod tests {
                     a: {
                         load: Clear,
                         store: DontCare,
-                        format: Format::R8G8B8A8Unorm,
+                        format: Format::R8G8B8A8_UNORM,
                         samples: 1,
                     }
                 },
@@ -932,11 +932,11 @@ mod tests {
         );
 
         let a = ImageView::new(
-            AttachmentImage::new(device.clone(), [256, 512], Format::R8G8B8A8Unorm).unwrap(),
+            AttachmentImage::new(device.clone(), [256, 512], Format::R8G8B8A8_UNORM).unwrap(),
         )
         .unwrap();
         let b = ImageView::new(
-            AttachmentImage::new(device.clone(), [256, 512], Format::R8G8B8A8Unorm).unwrap(),
+            AttachmentImage::new(device.clone(), [256, 512], Format::R8G8B8A8_UNORM).unwrap(),
         )
         .unwrap();
 
