@@ -65,7 +65,7 @@
 use crate::check_errors;
 use crate::device::Device;
 use crate::device::DeviceOwned;
-pub use crate::pipeline::depth_stencil::Compare;
+use crate::pipeline::depth_stencil::CompareOp;
 use crate::Error;
 use crate::OomError;
 use crate::VulkanObject;
@@ -77,7 +77,7 @@ use std::sync::Arc;
 
 /// Describes how to retrieve data from an image within a shader.
 pub struct Sampler {
-    sampler: ash::vk::Sampler,
+    handle: ash::vk::Sampler,
     device: Arc<Device>,
     compare_mode: bool,
     unnormalized: bool,
@@ -225,7 +225,7 @@ impl Sampler {
         max_anisotropy: f32,
         min_lod: f32,
         max_lod: f32,
-        compare: Compare,
+        compare: CompareOp,
     ) -> Result<Arc<Sampler>, SamplerCreationError> {
         Sampler::new_impl(
             device,
@@ -255,7 +255,7 @@ impl Sampler {
         max_anisotropy: f32,
         min_lod: f32,
         max_lod: f32,
-        compare: Option<Compare>,
+        compare: Option<CompareOp>,
     ) -> Result<Arc<Sampler>, SamplerCreationError> {
         assert!(max_anisotropy >= 1.0);
         assert!(min_lod <= max_lod);
@@ -266,10 +266,7 @@ impl Sampler {
                 return Err(SamplerCreationError::SamplerAnisotropyFeatureNotEnabled);
             }
 
-            let limit = device
-                .physical_device()
-                .properties()
-                .max_sampler_anisotropy;
+            let limit = device.physical_device().properties().max_sampler_anisotropy;
             if max_anisotropy > limit {
                 return Err(SamplerCreationError::AnisotropyLimitExceeded {
                     requested: max_anisotropy,
@@ -280,10 +277,7 @@ impl Sampler {
 
         // Check mip_lod_bias value.
         {
-            let limit = device
-                .physical_device()
-                .properties()
-                .max_sampler_lod_bias;
+            let limit = device.physical_device().properties().max_sampler_lod_bias;
             if mip_lod_bias > limit {
                 return Err(SamplerCreationError::MipLodBiasLimitExceeded {
                     requested: mip_lod_bias,
@@ -322,7 +316,7 @@ impl Sampler {
         };
 
         let fns = device.fns();
-        let sampler = unsafe {
+        let handle = unsafe {
             let infos = ash::vk::SamplerCreateInfo {
                 flags: ash::vk::SamplerCreateFlags::empty(),
                 mag_filter: mag_filter.into(),
@@ -366,7 +360,7 @@ impl Sampler {
         };
 
         Ok(Arc::new(Sampler {
-            sampler: sampler,
+            handle,
             device: device.clone(),
             compare_mode: compare.is_some(),
             unnormalized: false,
@@ -424,7 +418,7 @@ impl Sampler {
             (b, None) => b,
         };
 
-        let sampler = unsafe {
+        let handle = unsafe {
             let infos = ash::vk::SamplerCreateInfo {
                 flags: ash::vk::SamplerCreateFlags::empty(),
                 mag_filter: filter.into(),
@@ -458,7 +452,7 @@ impl Sampler {
         };
 
         Ok(Arc::new(Sampler {
-            sampler: sampler,
+            handle,
             device: device.clone(),
             compare_mode: false,
             unnormalized: true,
@@ -530,24 +524,33 @@ unsafe impl VulkanObject for Sampler {
 
     #[inline]
     fn internal_object(&self) -> ash::vk::Sampler {
-        self.sampler
+        self.handle
     }
 }
 
 impl fmt::Debug for Sampler {
     #[inline]
     fn fmt(&self, fmt: &mut fmt::Formatter) -> Result<(), fmt::Error> {
-        write!(fmt, "<Vulkan sampler {:?}>", self.sampler)
+        write!(fmt, "<Vulkan sampler {:?}>", self.handle)
     }
 }
 
+impl PartialEq for Sampler {
+    #[inline]
+    fn eq(&self, other: &Self) -> bool {
+        self.handle == other.handle
+    }
+}
+
+impl Eq for Sampler {}
+
 impl Drop for Sampler {
     #[inline]
     fn drop(&mut self) {
         unsafe {
             let fns = self.device.fns();
             fns.v1_0
-                .destroy_sampler(self.device.internal_object(), self.sampler, ptr::null());
+                .destroy_sampler(self.device.internal_object(), self.handle, ptr::null());
         }
     }
 }
@@ -851,7 +854,7 @@ mod tests {
             1.0,
             0.0,
             2.0,
-            sampler::Compare::Less,
+            sampler::CompareOp::Less,
         )
         .unwrap();
 
